/*
   Setup functions
   --------------------------------------------------------------------
   Linberto

   Copyright (C) 1999-2001 Diego Javier Grigna <diego@grigna.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"

struct config_file opt; /* It is the global option struct                  */
struct config_file tmp; /* If we cancel the changes, return to the default */

char *blockkind_str[ BLOCK_MAX + 1] = {
 gettext_noop("Plain ")    ,
 gettext_noop("Textured  "),
 gettext_noop("Wire      "),
 gettext_noop("Spheric  ") ,
};

char *enemykind_str[ ENEMY_MAX + 1] = {
 gettext_noop("Dumb  ") ,
 gettext_noop("Genius "),
};

/* Global setup 'x' and 'y' */
int gox, goy;

/* Used by setup_draw_bar */
void (* optbar_function)( double);

static void setup_printmenu( void);
static void setup_block_change( void);
static void setup_block_change_menu( void);
static double setup_bar( char *title , char *minstr,
                        char *maxstr, double curpos, double maxpos,
                        double inc);
static double setup_draw_bar( double pos, double maxpos);
static void setup_live( void);
static void setup_stage( void);
static void setup_speed( void);
void setup_live_bar( double lives);
void setup_speed_bar( double speed);
static void setup_stage_text( int stage);


void setup( void)
{
 int pos = 0;

 sound_music( SND_NONE);

 tmp.cfg_language     = opt.cfg_language;
 tmp.cfg_invincible   = opt.cfg_invincible;
 tmp.cfg_start_stage  = opt.cfg_start_stage;
 tmp.cfg_sound        = opt.cfg_sound;
 tmp.cfg_lives        = opt.cfg_lives;
 tmp.cfg_showtext     = opt.cfg_showtext;
 tmp.cfg_blockkind    = opt.cfg_blockkind;
 tmp.cfg_enemykind    = opt.cfg_enemykind;
 tmp.cfg_speed        = opt.cfg_speed;
 strcpy( tmp.cfg_world_path, opt.cfg_world_path);
 tmp.cfg_joystick     = opt.cfg_joystick;

 while( 1) {

        setup_printmenu();

        pos = menu( 4, 18, 142, pos, 14, 13);

        switch( pos) {
                /* Language Selection */
                case 0: sound_send( SND_CHANGE);
                        opt.cfg_language++;
                        if( opt.cfg_language > LAST_LANGUAGE)
                            opt.cfg_language = DEFAULT_LANGUAGE;
                        nls_change_language();
                        break;
                /* Invincible Mode */
                case 1: sound_send( SND_CHANGE);
                        opt.cfg_invincible = opt.cfg_invincible == INVINCIBLE_OFF ? INVINCIBLE_ON: INVINCIBLE_OFF;
                        break;
                /* Number of lives */
                case 2: sound_send( SND_CHANGE);
                        setup_live();
                        break;
                /* Starting Stage */
                case 3: sound_send( SND_CHANGE);
                        setup_stage();
                        break;
                /* Sound config */
                case 4: sound_send( SND_CHANGE);
                        opt.cfg_sound = opt.cfg_sound == SOUND_OFF ? SOUND_ON: SOUND_OFF;
                        break;
                /* Show text */
                case 5: sound_send( SND_CHANGE);
                        opt.cfg_showtext = opt.cfg_showtext == SHOWTEXT_ON ?  SHOWTEXT_OFF : SHOWTEXT_ON;
                        break;
                /* Block kind */
                case 6: sound_send( SND_CHANGE);
                        menu_spacing  = 30;
                        menu_function = &setup_block_change_drawcubes;
                        setup_block_change();
                        menu_spacing  = DEFAULT_MENU_SPACING;
                        menu_function = NULL;
                        break;
                /* Enemy kind */
                case 7: sound_send( SND_CHANGE);
                        opt.cfg_enemykind = opt.cfg_enemykind == ENEMY_DUMB ? ENEMY_GENIUS: ENEMY_DUMB;
                        break;
                /* Speed      */
                case 8: sound_send( SND_CHANGE);
                        setup_speed();
                        break;
                /* Joystick   */
                case 9: sound_send( SND_CHANGE);
                        opt.cfg_joystick = opt.cfg_joystick 
                                           == LIN_JOYSTICK_OFF ? 
                                           LIN_JOYSTICK_ON: LIN_JOYSTICK_OFF;
                        break;
                /* World file */
                case 10: world_filename[ 0] = 0;
                         world_select();
                         if( world_filename[ 0] != 0)
                             strcpy( opt.cfg_world_path, world_filename);
                         break;
                /* World editor */
                case 11: world_editor();
                         init_load_stage( opt.cfg_world_path);
                         break;
                /* Default settings */
                case 12: sound_send( SND_CHANGE);
                         opt.cfg_language     = DEFAULT_LANGUAGE;
                         opt.cfg_invincible   = DEFAULT_INVINCIBLE;
                         opt.cfg_start_stage  = DEFAULT_STAGE;
                         opt.cfg_sound        = DEFAULT_SOUND;
                         opt.cfg_lives        = DEFAULT_LIVES;
                         opt.cfg_showtext     = DEFAULT_SHOWTEXT;
                         opt.cfg_blockkind    = DEFAULT_BLOCKKIND;
                         opt.cfg_enemykind    = DEFAULT_ENEMYKIND;
                         opt.cfg_speed        = DEFAULT_SPEED;
                         strcpy( opt.cfg_world_path, LINBERTO_WORLDFILE);
                         opt.cfg_joystick     = DEFAULT_JOYSTICK;
                         setup_printmenu();
                         nls_change_language();
                         break;
                /* Cancel */
                case 13: opt.cfg_language    = tmp.cfg_language;
                         opt.cfg_invincible  = tmp.cfg_invincible;
                         opt.cfg_start_stage = tmp.cfg_start_stage;
                         opt.cfg_sound       = tmp.cfg_sound;
                         opt.cfg_lives       = tmp.cfg_lives;
                         opt.cfg_showtext    = tmp.cfg_showtext;
                         opt.cfg_blockkind   = tmp.cfg_blockkind;
                         opt.cfg_enemykind   = tmp.cfg_enemykind;
                         opt.cfg_speed       = tmp.cfg_speed;
                         strcpy( opt.cfg_world_path, tmp.cfg_world_path);
                         opt.cfg_joystick    = tmp.cfg_joystick;
                         nls_change_language();
                         return;
                /* Save */
                case 14: init_save_config();
                         nls_change_language();
                         return;
        } /* end switch( pos) */

 } /* end while( 1) */

}

static void setup_printmenu( void)
{
 int x, y;

 gl_clearscreen( 0);
 screen_center_text( _(" Configuration menu "), 0);
 lin_title_line();

 x =  5;
 y = 20;

 grf_printf( x, y      , _(" Language         : %s") , _("English"));
 grf_printf( x, y +  10, _(" Invincible Mode  : %s") , opt.cfg_invincible == INVINCIBLE_ON ? _("On "): _("Off"));
 grf_printf( x, y +  20, _(" Lives            : %d") , opt.cfg_lives);
 grf_printf( x, y +  30, _(" Starting Stage   : %-10d"), opt.cfg_start_stage);
 grf_printf( x, y +  40, _(" Sound            : %s"), opt.cfg_sound      == SOUND_ON      ? _("On "): _("Off"));
 grf_printf( x, y +  50, _(" Show text info.  : %s"), opt.cfg_showtext   == SHOWTEXT_ON   ? _("On "): _("Off"));
 grf_printf( x, y +  60, _(" Kind of block    : %s"), _(blockkind_str[ opt.cfg_blockkind]));
 grf_printf( x, y +  70, _(" Enemy kind       : %s"), _(enemykind_str[ opt.cfg_enemykind]));
 grf_printf( x, y +  80, _(" Speed            : x%-1.1f"), opt.cfg_speed);
 grf_printf( x, y +  90, _(" Joystick         : %s"), opt.cfg_joystick == LIN_JOYSTICK_ON ? _("On "): _("Off"));
 grf_printf( x, y + 100, _(" World file       : %s"), opt.cfg_world_path);
 gl_write(   x, y + 110, _(" World editor     "));
 gl_write(   x, y + 120, _(" Default settings "));
 gl_write(   x, y + 130, _(" Cancel           ")); 
 gl_write(   x, y + 140, _(" Save             ")); 
}

static void setup_block_change( void)
{
 int pos;

 pos = opt.cfg_blockkind;

 while( 1) {
        setup_block_change_menu();
        setup_block_change_drawcubes( pos);
        pos = menu( 54, 30, 90, pos, BLOCK_MAX + 1, BLOCK_MAX);

        switch( pos) {
                case 0: opt.cfg_blockkind = BLOCK_PLAIN;
                        return;
                case 1: opt.cfg_blockkind = BLOCK_TEXTURE;
                        return;
                case 2: opt.cfg_blockkind = BLOCK_WIRE;
                        return;
                case 3: opt.cfg_blockkind = BLOCK_SPHERE;
                        return;
                case 4: return;
        } /* end switch( pos) */
 } /* end while */
}

static void setup_block_change_menu( void)
{
 gl_clearscreen( 0);
 screen_center_text( _("Select kind of block"), 0);
 lin_title_line();

 gox = 60;
 goy = 32;

 gl_write( gox, goy      , _(blockkind_str[ BLOCK_PLAIN  ]));
 gl_write( gox, goy +  30, _(blockkind_str[ BLOCK_TEXTURE]));
 gl_write( gox, goy +  60, _(blockkind_str[ BLOCK_WIRE   ]));
 gl_write( gox, goy +  90, _(blockkind_str[ BLOCK_SPHERE ]));
 gl_write( gox, goy + 120, _("Cancel  "));
}

void setup_block_change_drawcubes( int pos)
{
 if( pos == 0)
     draw_cube(         gox + 120, goy -  9, lcolors[ 3]);
 else
     draw_cube(         gox + 120, goy -  9, lcolors[ 0]);

 if( pos == 1)
     draw_texture_cube( gox + 120, goy + 21, lcolors[ 3]);
 else
     draw_texture_cube( gox + 120, goy + 21, lcolors[ 0]);

 if( pos == 2)
     draw_wire_cube(    gox + 120, goy + 51, lcolors[ 3]);
 else
     draw_wire_cube(    gox + 120, goy + 51, lcolors[ 0]);

 if( pos == 3)
     draw_sphere(       gox + 120, goy + 82, lcolors[ 3]);
 else
     draw_sphere(       gox + 120, goy + 82, lcolors[ 0]);
}

static void setup_live( void)
{
 char buf[ 64];
 int temp;

 sprintf( buf, "%d", LIVES_MAX);

 optbar_function = &setup_live_bar;

 temp = ( int) setup_bar( _("Select the number of lives"), 
                            "0", buf, opt.cfg_lives, LIVES_MAX, 1);

 if( temp != -1)
     opt.cfg_lives = temp;

 if( opt.cfg_lives == 0)
     opt.cfg_lives = 1;
}

static double setup_bar( char *title , char *minstr,
                        char *maxstr, double curpos, double maxpos,
                        double inc)
{
 double pos = curpos;

 gl_clearscreen( 0);
 screen_center_text( title, 0);
 lin_title_line();

 screen_center_text( _("Press 'ENTER' to accept"), 165);
 screen_center_text( _("Or 'ESC' to cancel")     , 175);

 gl_write( 61,  18, minstr);
 gl_write( 61, 148, maxstr);

 pos = setup_draw_bar( pos, maxpos);

 while( 1) {
       if( keyboard_update() == 0 && joy_moved() == 0)
           continue;

       switch( lin_scancode) {
          case SCANCODE_KEYPADENTER:
          case SCANCODE_ENTER:
                 return pos;
          case SCANCODE_Q:
          case SCANCODE_ESCAPE:
                 return -1;
          case SCANCODE_CURSORLEFT:
          case SCANCODE_CURSORBLOCKLEFT:
          case SCANCODE_CURSORUP:
          case SCANCODE_CURSORBLOCKUP:
                 sound_send( SND_MENU);
                 pos -= inc;
                 pos = setup_draw_bar( pos, maxpos);
                 continue;
          case SCANCODE_CURSORRIGHT:
          case SCANCODE_CURSORBLOCKRIGHT:
          case SCANCODE_CURSORDOWN:
          case SCANCODE_CURSORBLOCKDOWN:
                 sound_send( SND_MENU);
                 pos += inc;
                 pos = setup_draw_bar( pos, maxpos);
                 continue;
          case SCANCODE_HOME:
          case SCANCODE_KEYPAD7:
                 sound_send( SND_MENU);
                 pos = setup_draw_bar( 0, maxpos);
                 continue;
          case SCANCODE_END:
          case SCANCODE_KEYPAD1:
                 sound_send( SND_MENU);
                 pos = setup_draw_bar( maxpos, maxpos);
                 continue;
       } /* end switch */
 } /* end while( 1) */

}

static double setup_draw_bar( double pos, double maxpos)
{
 int    i;
 int    x;
 int    y;
 int maxx;
 int bars; /* Bar size */
 int porc;

 x    =  20;
 y    =  20;
 maxx = 129;
 bars =  30;

 draw_rectangle( x, y, bars + 4, maxx + 4, LIN_RED);

 for( i = y; i < y + maxx + 4; i += 6)
      gl_line( x + bars + 4, i, x + bars + 6, i, LIN_WHITE);

 if( pos > maxpos) pos = maxpos;
 if( pos <      0) pos =      0;

 porc = maxx * pos / maxpos;

 /* Draw the bars */
 gl_fillbox( x + 2, y + 2       , bars,        porc, LIN_WHITE);
 gl_fillbox( x + 2, y + 2 + porc, bars, maxx - porc, LIN_BLACK);

 if( optbar_function != NULL)
     optbar_function( pos);

 return pos;
}

void setup_live_bar( double lives)
{
 char buf[ 128];
 int x, y;
 int len;

 x = 80;
 y = 75;
 sprintf( buf, _("Lives %2d"), ( int) lives);
 len = strlen( buf) * 8;

 draw_rectangle( x - 6, y - 5, len + 10, 17, LIN_RED);
 gl_write( x, y, buf);
}

static void setup_stage( void)
{
 FILE *fp;
 char filename[ 8192];
 struct world_header whdr;
 int curr_stage;

 sprintf( filename, "%s/%s", LINBERTO_DATADIR, opt.cfg_world_path);

 fp = linberto_openfile( filename, LINBERTO_WORLDFILE_DEFAULT, "r");

 fread( &whdr, sizeof( struct world_header), 1, fp);

 if( strcmp( whdr.magic, WORLD_MAGIC)) {
     fprintf( stderr, _("\nThe file [%s] isn't a Linberto world file.\n\n"), openedfile);
     exit_game( 1);
 }

 curr_stage = opt.cfg_start_stage;

 fseek( fp, ( long) sizeof( struct world_header) + curr_stage *
                    sizeof( map_stage), SEEK_SET);

 fread( &map_stage, sizeof( map_stage), 1, fp);

 gl_clearscreen( 0);
 setup_stage_text( curr_stage); 

 while( 1) {
        if( keyboard_update() == 0 && joy_moved() == 0)
            continue;

        switch( lin_scancode) {
           case SCANCODE_KEYPADENTER:
           case SCANCODE_ENTER:
                  opt.cfg_start_stage = curr_stage;
                  fclose( fp);
                  return;
           case SCANCODE_Q:
           case SCANCODE_ESCAPE:
                  fclose( fp);
                  return;
           case SCANCODE_CURSORLEFT:
           case SCANCODE_CURSORBLOCKLEFT:
           case SCANCODE_CURSORUP:
           case SCANCODE_CURSORBLOCKUP:
                  if( curr_stage < 1)
                      continue;
                  sound_send( SND_BLKN);
                  curr_stage--;
                  fseek( fp, (long) -2 * sizeof( map_stage), SEEK_CUR);
                  fread( &map_stage, sizeof( map_stage), 1, fp);
                  setup_stage_text( curr_stage);
                  continue;
           case SCANCODE_CURSORRIGHT:
           case SCANCODE_CURSORBLOCKRIGHT:
           case SCANCODE_CURSORDOWN:
           case SCANCODE_CURSORBLOCKDOWN:
                  if( curr_stage >= whdr.number - 1)
                      continue;
                  sound_send( SND_BLKN);
                  curr_stage++;
                  fread( &map_stage, sizeof( map_stage), 1, fp);
                  setup_stage_text( curr_stage);
                  continue;
           case SCANCODE_HOME:
           case SCANCODE_KEYPAD7:
                  sound_send( SND_BLKN);
                  fseek( fp, (long) sizeof( struct world_header), SEEK_SET);
                  fread( &map_stage, sizeof( map_stage), 1, fp);
                  curr_stage = 0;
                  setup_stage_text( curr_stage);
                  continue;
           case SCANCODE_END:
           case SCANCODE_KEYPAD1:
                  sound_send( SND_BLKN);
                  fseek( fp, (long) -1 * sizeof( map_stage), SEEK_END);
                  fread( &map_stage, sizeof( map_stage), 1, fp);
                  curr_stage = whdr.number - 1;
                  setup_stage_text( curr_stage);
                  continue;
        } /* end switch */
 } /* end while */
}

static void setup_speed( void)
{
 char bufmin[ 64];
 char bufmax[ 64];
 double temp;

 sprintf( bufmin, "%.2f", SPEED_MIN);
 sprintf( bufmax, "%.2f", SPEED_MAX);

 optbar_function = &setup_speed_bar;

 temp = setup_bar( _("Select the speed"),
                     bufmin, bufmax, opt.cfg_speed, SPEED_MAX, 0.1);

 if( temp != -1)
     opt.cfg_speed = temp;
}

void setup_speed_bar( double speed)
{
 char buf[ 128];
 int x, y;
 int len;

 x = 80;
 y = 75;
 sprintf( buf, _("Speed %2.1f"), speed);
 len = strlen( buf) * 8;

 draw_rectangle( x - 6, y - 5, len + 10, 17, LIN_RED);
 gl_write( x, y, buf);
}

static void setup_stage_text( int stage)
{
 char buf[ 128];

 gl_fillbox( 88, 30, 128, 104, LIN_BLACK);

 screen_center_text( _("Select the starting stage"), 0);
 lin_title_line();

 sprintf( buf, _("Stage %d"), stage);
 screen_center_text( buf, 125);

 screen_center_text( _("\x18 \x19 \x1A \x1B change stage"), 150);
 screen_center_text( _("ENTER to accept")                 , 160);
 screen_center_text( _("ESC to cancel")                   , 170);

 draw_mini_board( 96, 30);
}


